﻿using GDWeave;
using GDWeave.Godot;
using GDWeave.Godot.Variants;
using GDWeave.Modding;

namespace PoppingBottles;

public class Mod : IMod {

    public Mod(IModInterface modInterface) {
        modInterface.Logger.Information("Loaded PoppingBottles!");
        
        modInterface.RegisterScriptMod(new Patch());
    }

    public void Dispose() {
        // Cleanup anything you do here
    }
}

public class Patch : IScriptMod
{
    public bool ShouldRun(string path) => path == "res://Scenes/Entities/Player/player.gdc";

    public IEnumerable<Token> Modify(string path, IEnumerable<Token> tokens)
    {
        var functionWaiter = new FunctionWaiter("_consume_item");

        var growthWaiter = new MultiTokenWaiter([
            t => t is ConstantToken { Value: StringVariant { Value: "growth" } },
            t => t.Type is TokenType.Colon,
            t => t is IdentifierToken { Name: "player_scale" },
            t => t.Type is TokenType.OpAssign,
            t => t.Type is TokenType.BuiltInFunc,
            t => t.Type is TokenType.ParenthesisOpen,
            t => t is IdentifierToken { Name: "player_scale" },
            t => t.Type is TokenType.OpAdd,
            t => t is ConstantToken { Value: RealVariant { Value: 0.1 } },
            t => t.Type is TokenType.Comma,
            t => t is ConstantToken { Value: RealVariant { Value: 0.7 } },
            t => t.Type is TokenType.Comma,
            t => t is ConstantToken { Value: RealVariant { Value: 1.3 } },
        ], true, true);
        
        var shrinkWaiter = new MultiTokenWaiter([
            t => t is ConstantToken { Value: StringVariant { Value: "shrink" } },
            t => t.Type is TokenType.Colon,
            t => t is IdentifierToken { Name: "player_scale" },
            t => t.Type is TokenType.OpAssign,
            t => t.Type is TokenType.BuiltInFunc,
            t => t.Type is TokenType.ParenthesisOpen,
            t => t is IdentifierToken { Name: "player_scale" },
            t => t.Type is TokenType.OpSub,
            t => t is ConstantToken { Value: RealVariant { Value: 0.1 } },
            t => t.Type is TokenType.Comma,
            t => t is ConstantToken { Value: RealVariant { Value: 0.7 } },
            t => t.Type is TokenType.Comma,
            t => t is ConstantToken { Value: RealVariant { Value: 1.3 } },
        ], true, true);

        foreach (var token in tokens)
        {
            if (functionWaiter.Check(token))
            {
                growthWaiter.SetReady();
                shrinkWaiter.SetReady();
            }

            if (growthWaiter.Check(token) || shrinkWaiter.Check(token))
            {
                yield return new ConstantToken(new RealVariant(100.0));
            }
            else yield return token;
        }
    }
}